%   ELAB_SONOGRAMA.M
%   Entradas:
%       SONOGRAM_DATA.MAT: Lneas de radiofrecuencia de pre y
%       postcompresin.
%     
%   Autor: Daniel Serra Freese
%   e-mail: danielserrafreese@gmail.com
%
%   Ultima resvision 7 septiembre 2016

%% Cargar datos
% Limpiamos espacio de trabajo
close all
clear corrAmounts
clear maxLocations

% Cargamos datos
load phantom_parameters.mat
load sonogram_parameters.mat
load sonogram_data.mat


% Paramatros de postprocesado
overlapping = 80; % [%]
windowSize = 5/1000; %[mm]

% Calcula solapamiento
overlapping= 1-overlapping/100;


%% Cortamos las señales (imágenes)
% En este paso se cortan las señales para dejar la parte útil unicamente.

% Para cada caso (PRE y POST) obtenemos la longitud de la señal en muestras
% y posteriormente cortamos de 1 hasta dicho número de muestras.
signal_pre = rfAllLinesAligned_pre;
signal_post = rfAllLinesAligned_post;

nCut_pre = round((z_size/c)*fs*2);% Muestra hasta la que tenemos que cortar
nCut_post = round((z_size-compresion)/c*fs*2);

signal_pre= signal_pre(1:nCut_pre, :); % Cortamos hasta dicha muestra
signal_post = signal_post(1:nCut_pre,:); % Igual tamaño (a la mayor)





%% Recortamos senales y las estiramos (stretching)

% P = nCut_pre;
% Q = nCut_post;
%
% envCuttedResampled_post = resample(envCut_post,P,Q); % Resampled

% %
% % signal_pre = envCut_pre;
% %
% %
% % signal_post = zeros(length(envCut_pre),no_lines);
% % signal_post(1:length(envCut_pre),1:no_lines) = envCut_post;
% %


%% Localizacion desplazamientos

nWindowSize = round(windowSize*fs/c); % Tamano del kernel
step = round(nWindowSize*overlapping); %Fijamos pasos entre correlaciones

windowShifter = 0;
indices = -nWindowSize:1:nWindowSize; % Para plot

for currentLine=1:no_lines
    disp(['Calculando linea ' num2str(currentLine) ' de ' num2str(no_lines) '.'])
    
    i=1;
    maxLocationBefore = 0;
    maxLocationBeforeBefore= 0;
    maxLocationBeforeBeforeBefore= 0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Ventanas
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % Ventana fija (0, 1, 1) Descartamos retrasos (>0)
    z1 = zeros(1, nWindowSize);
    o1 = ones(1, nWindowSize);
    wFija = [z1 1 o1]; 
    
    % Ventana desplazable (0, HANNING, 0)
    z2 = zeros(1,round((2*nWindowSize+1)*0.3)); %% 30 porciento Zeros
    h2 = hanning(round((2*nWindowSize+1)*0.40))'; %% 40 porcioento Hanning
    z3 = zeros(1,round((2*nWindowSize+1)*0.3),1); %% 30 porciento Zeros
    wTemporal = [z2 h2 z3];
    
    if length(wTemporal) > (2*nWindowSize+1)
        % disp('MAYOR')
        wDesplazable = wTemporal(1:(2*nWindowSize+1));
    elseif length(wTemporal) < (2*nWindowSize+1)
        % disp('MENOR')
        wDesplazable = zeros(1,(2*nWindowSize+1));
        wDesplazable(1:length(wTemporal)) = wTemporal;
    elseif length(wTemporal) == (2*nWindowSize+1)
        % disp('IGUAL')
        wDesplazable = wTemporal
    end
    %%%%%%%%%%%%% E % N % D %%%%%%%%%%%%%%%%
    
    
    for wStart=1:step:length(signal_pre(:,currentLine))-1-nWindowSize
        
        x = signal_pre(wStart:wStart+nWindowSize,currentLine);
        y = signal_post(wStart:wStart+nWindowSize,currentLine);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Shifting window
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        z2 = zeros(1,round((2*nWindowSize+1)*0.4));
        h2 = hanning(round((2*nWindowSize+1)*0.2))';
        z3 = zeros(1,round((2*nWindowSize+1)*0.40),1);
        zShift = zeros(1,maxLocationBefore);
        wTemporal = [z2 zShift h2 z3];
        
        if length(wTemporal) > (2*nWindowSize+1)
            % disp('MAYOR')
            wDesplazable = wTemporal(1:(2*nWindowSize+1));
        elseif length(wTemporal) < (2*nWindowSize+1)
            % disp('MENOR')
            wDesplazable = zeros(1,(2*nWindowSize+1));
            wDesplazable(1:length(wTemporal)) = wTemporal;
        elseif length(wTemporal) == (2*nWindowSize+1)
            % disp('IGUAL')
            wDesplazable = wTemporal;
        end
        %%%%%%%%%%%%% E % N % D %%%%%%%%%%%%%%%%
        
        
        currentCorrelation = xcorr(x,y); % Columna
        %         currentCorrelation = currentCorrelation.*hanning(2*nWindowSize+1); % Columna .x Columna
        currentCorrelation = currentCorrelation.*wFija';
        currentCorrelation = currentCorrelation.*wDesplazable';
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Plot
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         plot(indices, [(wFija'.*hanning(2*nWindowSize+1)) wDesplazable' currentCorrelation/max(currentCorrelation)]),drawnow, hold off
%         axis([-nWindowSize nWindowSize 0 1])
%         drawnow
%         pause(0.3)
        %%%%%%%%%%%% E % N % D %%%%%%%%%%%%%%%%
        
        
        [corrAmount, maxLocation]=max(currentCorrelation);
        corrAmounts(i,currentLine)=corrAmount; % Vector
        
        maxLocation = maxLocation-nWindowSize;
        maxLocations(i,currentLine)=maxLocation; % Vector
        
        tolerancia = 5;
        
        if (maxLocation < maxLocationBefore);
            
            maxLocationBefore = maxLocationBefore;
            enhancedLocations(i,currentLine) = maxLocationBefore;
            
        elseif (maxLocation >= maxLocationBefore) & (abs(maxLocationBeforeBeforeBefore-maxLocation) < 5);;
            
            maxLocationBefore = maxLocation;
            enhancedLocations(i,currentLine) = maxLocation;
        
        end
        
        maxLocationBeforeBefore= maxLocationBefore;
        maxLocationBeforeBeforeBefore= maxLocationBeforeBefore;
        
        %         % PLOT progreso
        %         subplot(2,2,1)
        %         plot(x,'r'),hold on, plot(y,'g'),hold off
        %
        %         subplot(2,2,2)
        %         plot(-nWindowSize:nWindowSize,currentCorrelation)
        %
        %         subplot(2,2,3:4)
        %         plot(maxLocations);
        %         drawnow
        %
        %         hold off
        
        i=i+1;
    end
end

disp(['Tamaño de la ventana de correlación: ' num2str(nWindowSize)])
disp(['Número de desplazamientos obtenidos: ' num2str(i)])
i

%
tiempo=0.00;


%% Temporal
close all
figure
len = i;
subplot(1,2,1)
pcolor(maxLocations), shading flat
title('ORIGINAL MAXLOCATIONS')

% subplot(1,3,1)
% for i=1:no_lines
% 
% plot(maxLocations(:,i),'r')
% axis([1  length(maxLocations) 0 70]); title(['Linea: ' num2str(i)]); 
% drawnow;
% 
% pause(tiempo)
% 
% 
% end


%% Enhanced locations

figure
subplot(1,3,2)
pcolor(enhancedLocations), shading flat
title('ENHANCEDMAXLOCATIONS')

% subplot(1,2,2)
% for i=1:no_lines
%     
%     plot(enhancedLocations(:,i),'r')
%     axis([1  length(enhancedLocations) 0 70]); title(['Linea: ' num2str(i)]);
%     drawnow;
%     
%     pause(tiempo)
%     
% end


%% Localizaciones normalizadas

for i=1:no_lines
    maximoDeUnaLinea = max(enhancedLocations(:,i));
    todaLaLinea = enhancedLocations(:,i);
    lineaNormalizada = todaLaLinea/maximoDeUnaLinea;
    localizacionesNormalizadas(:,i)=lineaNormalizada;
end

subplot(1,3,3)
pcolor(localizacionesNormalizadas), shading flat
title('NORMALIZEDMAXLOCATIONS')

% subplot(1,2,2)
% for i=1:no_lines
%     
%     plot(localizacionesNormalizadas(:,i),'r')
%     axis([1  length(localizacionesNormalizadas) 0 70]); title(['Linea: ' num2str(i)]);
%     drawnow;
%     
%     pause(tiempo)
%     
% end


%% Limpiamos elastogramas

interpolatedMatrix=zeros(length(maxLocations),no_lines);

for k=1:no_lines
    displacementLine=enhancedLocations(:,k);
    correctedDisplacementLine = displacementLine;
    
    % STEP 1: Remove negative displacements
    correctedDisplacementLine(find(displacementLine < 0)) = 0;
    
    
    % STEP 2
    pl=find(sign(diff(displacementLine))==-1);
    pl=pl;
    
    correctedDisplacementLine(pl) = 0; % Elimina esta
%    correctedDisplacementLine(pl+1) = 0; Elimina la siguiente
    

    index = find(correctedDisplacementLine ~=0);
    values = correctedDisplacementLine(index);
    
    
    
    interpolatedLine = interp1(index,values,1:1:length(displacementLine)); %Interpolamos para el siguiente paso
    interpolatedLine = smooth(interpolatedLine,3);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Plot
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     stem(displacementLine,'r'), hold on
%     stem(interpolatedLine,'g'),hold off
%     drawnow;pause(2)
    %%%%%%%%%%%%% E % N % D %%%%%%%%%%%%%%%%
        
    interpolatedMatrix(1:length(correctedDisplacementLine),k)=interpolatedLine;
 
end

%%

subplot(1,2,1)
surf(maxLocations/max(max(maxLocations)))
colormap jet, shading interp

subplot(1,2,2)
surf(interpolatedMatrix/max(max(interpolatedMatrix)))
colormap jet, shading interp

%% Guarda datos
save elastogramInfo.mat corrAmounts maxLocations enhancedLocations

%% Representa
%
% % PREcompresión
% figure
% subplot(1,2,1)
% envCutdB_pre=20*log10(envCut_pre); % Pasamos a dB
% envCutdB_pre=envCutdB_pre-max(max(envCutdB_pre)); %Normalizamos a 0 dB
% envCutGray_pre=127*(env_dB_pre+60)/60; % Rango de unos 60 dB
%
% depth=((0:size(envCut_pre,1)))/fs*c/2;  % Se ha quitado un /2 debido al ToF
% x=((1:no_lines)-no_lines/2)*dx;
%
% image(x*1000, depth*1000, env_gray_pre)
% xlabel('Profundidad [mm]')
% axis('image')
% colormap(gray(128))
% title('PREcompresion')
%
% % POSTcompresión
% subplot(1,2,2)
% env_dB_post=20*log10(env_post); % Pasamos a dB
% env_dB_post=env_dB_post-max(max(env_dB_post)); % Normalizamos a 0 dB
% env_gray_post=127*(env_dB_post+60)/60; % Rango de unos 60 dB
% 
% depth=((0:size(envCut_post,1)))/fs*c/2;
% x=((1:no_lines)-no_lines/2)*dx;
% 
% image(x*1000, depth*1000, env_gray_post)
% xlabel('Profundidad [mm]')
% axis('image')
% colormap(gray(128))
% title('POSTcompresion')
